//
//  AccessMethodRepository+Stub.swift
//  MullvadRESTTests
//
//  Created by Mojgan on 2024-01-02.
//  Copyright © 2024 Mullvad VPN AB. All rights reserved.
//

import Combine
import MullvadSettings

public struct AccessMethodRepositoryStub: AccessMethodRepositoryDataSource {
    public var directAccess: PersistentAccessMethod

    public var accessMethodsPublisher: AnyPublisher<[PersistentAccessMethod], Never> {
        passthroughSubject.eraseToAnyPublisher()
    }

    let passthroughSubject: CurrentValueSubject<[PersistentAccessMethod], Never> = CurrentValueSubject([])

    public init(accessMethods: [PersistentAccessMethod]) {
        directAccess = accessMethods.first(where: { $0.kind == .direct })!
        passthroughSubject.send(accessMethods)
    }

    public func fetchAll() -> [PersistentAccessMethod] {
        passthroughSubject.value
    }

    public func saveLastReachable(_ method: PersistentAccessMethod) {}

    public func fetchLastReachable() -> PersistentAccessMethod {
        directAccess
    }
}
